//-*-C++-*- Header file for the AudDB Class
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Artistic License, included as the file
// "LICENSE" in the source code archive.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// You should have received a copy of the file "LICENSE", containing
// the License John Weiss originally placed this program under.
//
// This file is unreadable, because the comment-docifier is
// brainedead.  Try a "make srcdocs" to generate the
// HTML-documentation.
//
#ifndef _AudDB_h_
#define _AudDB_h_
#include "Catalog.h"
#include <string.h>
#include <iostream.h>
#include <fstream.h>


#define IOBUFLEN 256


/**
  {\bf AudDB} is a base class for creating databases of \Ref{Catalog}
  objects.

  @memo Read and write generic audio databases.
  @see Catalog

  */
class AudDB
{
   public:
      /** Constructor for the AudDB object.
	   Sets all poiners and counters to 0.
	   @memo
	  */
	 AudDB(void);

      /** Overloaded Constructor.  Difference:  \ref{db_name} is set to
	   a duplicate of the argument {\em name}.
	   @memo
	  */
	 AudDB(char *name);

      /** Destructor for the AudDB object.
	   @memo
	   */
	 virtual ~AudDB(void);

	 /**
	   Sets \Ref{db_name} to a copy of {\em name}.
	   @memo
	   */
	 void setName(char *name);

      /**
	   Adds the catalog {\em ct} to the database, overwriting any
	   existing entry for {\em ct}.
	   @memo
	   */
	 void add(Catalog &ct);

	 /** Abstract function for opening the database.
	   @memo
	   */
	 virtual void open(void) = 0;

	 /** {\tt [virtual]}

	   Reads a catalog entry, {\em ct}.
	   Performs a \Ref{find}(), then calls its overloaded version
	   using the index from the \Ref{find}().  Returns {\em false} if
	   it can't find {\em ct} in the database.
	   @memo
	   */
	 virtual bool read(Catalog& ct);


   protected:
	 /** 
	   A string containing the filename of the database.  If it is a
	   null string, the member methods all return an error.
	   @memo
	   */
	 char *db_name;
	 /** List of magic numbers.  Should be set up by {\bf open()}.
	   @memo
	   */
	 unsigned long *magiclist;
	 /** 
	   Number of elements in {\bf magiclist}.  Also number of entries
	   in the database.
	   @memo
	   */
	 int len_magic;

	 /** 
	   Searches for the catalog {\em ct} in the database.
	   @return a location index or, -1 if {\em ct} is not in the
	   database. 
	   @memo
	   */
	 int find(Catalog& ct);

	 /** 
	   Abstract function to write a catalog entry, {\em ct}, to the
	   database.  It should update \Ref{magiclist} and set {\em
	   oldlen} to the new value of \Ref{len_magic}.
	   @memo
	   */
	 virtual void write(Catalog& ct, int& oldlen) = 0;

	 /** 
	   Abstract function.  Searches for the {\em idx}th entry in
	   the database.  Returns {\em false} if {\em idx} is out of
	   range, if that entry doesn't exist, or if there is a read
	   error.  
	   @memo 
	   */
	 virtual bool read_i(Catalog& ct, int idx) = 0;

	 /** 
	   Abstract function to read a single entry, {\em ct}, from the
	   open stream {\em instr}.  Returns {\em true} on success.
	   @memo
	   */
	 virtual bool readEntry(ifstream& instr, Catalog& ct) = 0;

	 /** 
	   Abstract function to write a single entry, {\em ct}, to the
	   open stream {\em outstr}.  Returns {\em true} on success.
	   @memo
	   */
	 virtual bool writeEntry(ofstream& outstr, Catalog& ct) = 0;
};



#endif // _AudDB_h_
//||||||||||||||||||||||||||||||||
//
// End
// 
