//-*-C++-*- Header file for the pDisplay Class
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
// This file is unreadable, because the comment-docifier is
// brainedead.  Try a "make srcdocs" to generate the
// HTML-documentation.
//
#ifndef _pDisplay_h_
#define _pDisplay_h_ 1
#include <qframe.h>
#include "PixData.h"
#include "QMeter.h"
#include "QExtended.h"
#include "paneldefs.h"





/**
  The {\bf pDisplay} object is a colection of Qt-widgets for
  displaying all sorts of information.  The widgets themselves are all
  private members of the class, but we'll describe them here.

  There are two \Ref{QeLCDNumber}s, one for the track number and
  another for the time display.  Associated with these are a number of
  \Ref{QeLabel}s for displaying the play state, the play mode, and the
  time display mode.  There are also two \Ref{QeLabel}s for the
  program title and the track label.  Lastly, a \Ref{QMeter} displays
  the volume level.

  Both the time and track \Ref{QeLCDNumber}s have their own press- and
  release-signals in this class.  The \Ref{QeLabel}s all emit generic
  press- and release-signals for the time being.

  {\bf
  \begin{verbatim}
  
  ----------------------------------------------------------
  ----------------------------------------------------------
  \end{verbatim}
  }

  @memo Special {\bf QFrame} for displaying player info.
  @see QeLCDNumber, QeLabel, QMeter, PixData

  */
class pDisplay : public QFrame
{
	 //
	 Q_OBJECT

   public:
	 /**
	   Class Constructor.
	   @param parent The parent object of this widget.
	   @param dispfgcol the default foreground color for the chil
	   widgets of the {\bf pDisplay}.
	   @param ifn The font to use for the title and track labels
	   @param scale The scale factor for the mode icons.
	   @param compact Flag to indicate label styles.  {\em
	   compact=true} tells {\bf pDisplay} to use shorten the size of
	   the track and title labels.
  	   @memo
	   */
	 pDisplay( QWidget* parent, QColor& dispfgcol,
			 QFont& ifn, float scale=1.0, bool compact=false );

	 /**
	   Class Destructor
	   @memo
	   */
	 ~pDisplay( void );

	 /**
	   Moves the object, performing some internal bookkeeping at the
	   same time.
	   @memo
	   */
	 inline void moveAll( int x, int y );

	 /**
	   Overloaded version.
	   @memo
	   */
	 inline void moveAll( QPoint pos );

	 /**
	   @return The point in the x-direction where this widget ends.
	   @memo
	   */
	 inline QPoint xEndPt();

	 /**
	   @return The point in the y-direction where this widget ends.
	   @memo
	   */
	 inline QPoint yEndPt();
	 //
	 void setChildForegroundColor( QColor& fgc );
	 inline void useTitleLabel(void);
	 inline void useArtistLabel(void);

	 /**
	   Changes the palette of this object to {\em p}.
	   @memo
	   */
	 void changePalette( QPalette& p );

	 /**
	   Change the font of the title and track labels to {\em f}.
	   @memo
	   */
	 void changeDTextFont( QFont& f );
	 /**
	   Change the font of the time mode label to {\em f}.  This could
	   have unexpected effects; you're better off using the default.
	   @memo
	   */
	 void changeTDMFont( QFont& f );

	 //
   public slots:
	 /**
	   {\em Public Slot}

	   Displays the new time specified in the argument.  The time
	   should be in seconds.
	   @memo [{\tt public slot}]
	   */
	 void newTime(int);

	 /**
	   {\em Public Slot}

	   Displays the new track number specified in the argument.
	   @memo [{\tt public slot}]
	   */
	 void newTrack(int);

	 /**
	   {\em Public Slot}

	   Displays the new track label specified as an argument.
	   @memo [{\tt public slot}]
	   */
	 void newTrackName(char *);

	 /**
	   {\em Public Slot}

	   Displays the new title label specified as an argument.	   
	   @memo [{\tt public slot}]
	   */
	 void newTitleName(char *);

	 /**
	   {\em Public Slot}

	   Changes the play mode and play state icons, depending on the
	   value of the argument.  The argument should be a
	   \Ref{PlayMode} {\bf enum} OR-ed with a \Ref{PlayState} {\bf
	   enum}.
	   @memo [{\tt public slot}]
	   */
	 void newPlayMode(int);

	 /**
	   {\em Public Slot}

	   Changes the time mode label, depending on the value of the
	   argument.  The argument should be a \Ref{TimeMode} {\bf
	   enum}.
	   @memo [{\tt public slot}]
	   */
	 void newTimeMode(int);

	 /**
	   {\em Public Slot}

	   Changes the volume meter to display the new volume specified
	   as an argument to this slot.
	   @memo [{\tt public slot}]
	   */
	 void newVolume(float);


	 //
   signals:
	 /**
	   {\em Signal}

	   Emitted when a mouse press-event occurs on the time LCD.
	   Contains the current mouse position, in global screen
	   coordinates.
   	   @memo [{\tt signal}]
	   @see timeReleased, pressed
	   */
	 void timePressed(QPoint);

	 /**
	   {\em Signal}

	   Emitted when a mouse release-event occurs on the time LCD.
   	   @memo [{\tt signal}]
	   @see timePressed, released
	   */
	 void timeReleased();

	 /**
	   {\em Signal}

	   Emitted when a mouse press-event occurs on the track LCD.
	   Contains the current mouse position, in global screen
	   coordinates.
   	   @memo [{\tt signal}]
	   @see trackReleased, pressed
	   */
	 void trackPressed(QPoint);

	 /**
	   {\em Signal}

	   Emitted when a mouse release-event occurs on the track LCD.
   	   @memo [{\tt signal}]
	   @see trackPressed, released
	   */
	 void trackReleased();

	 /**
	   {\em Signal}

	   Emitted when a mouse press-event occurs on the box displaying
	   the track name.  Contains the current mouse position, in
	   global screen coordinates.
   	   @memo [{\tt signal}]
	   @see tracknameReleased, pressed */
	 void tracknamePressed(QPoint);

	 /**
	   {\em Signal}

	   Emitted when a mouse release-event occurs on the box displaying
	   the track name.
   	   @memo [{\tt signal}]
	   @see tracknamePressed, released
	   */
	 void tracknameReleased();

	 /**
	   {\em Signal}

	   Emitted when a mouse press-event occurs on the box displaying
	   the title.  Contains the current mouse position, in global
	   screen coordinates.
   	   @memo [{\tt signal}]
	   @see titleReleased, pressed */
	 void titlePressed(QPoint);

	 /**
	   {\em Signal}

	   Emitted when a mouse release-event occurs on the box displaying
	   the title.
   	   @memo [{\tt signal}]
	   @see titlePressed, released
	   */
	 void titleReleased();

	 /**
	   {\em Signal}

	   Emitted when a mouse press-event occurs on the mode light.
	   Contains the current mouse position, in global screen
	   coordinates.
   	   @memo [{\tt signal}]
	   @see modeReleased, pressed
	   */
	 void modePressed(QPoint);

	 /**
	   {\em Signal}

	   Emitted when a mouse release-event occurs on the mode light.
   	   @memo [{\tt signal}]
	   @see modePressed, released
	   */
	 void modeReleased();

	 /**
	   {\em Signal}

	   Emitted when a mouse press-event occurs anywhere else on the
	   panel outside of one of the widgets with their own
	   press-signal.  Contains the current mouse position, in global
	   screen coordinates.
   	   @memo [{\tt signal}]
	   @see timePressed, trackPressed, tracknamePressed,
	   titlePressed, modePressed */
	 void pressed(QPoint);

	 /**
	   {\em Signal}

	   Emitted when a mouse release-event occurs anywhere else on the
	   panel outside of one of the widgets with their own
	   release-signal.
   	   @memo [{\tt signal}]
	   @see timeReleased, trackReleased, tracknameReleased,
	   titleReleased, modeReleased
	   */
	 void released();

	 /**
	   {\em Signal}

	   Emitted when a mouse move-event occurs anywhere else on the
	   panel outside of one of the widgets.  Contains the current
	   mouse position, in global screen coordinates.
   	   @memo [{\tt signal}]
	   */
	 void moving(QPoint);


	 //
   private slots:
	 void childPressed(int,int);
	 void childReleased(int,int);
	 void childPressed(QPoint);
	 void childReleased(QPoint);

   private:
	 void createChildren( const char *boxwidthstr );
	 void sizeChildren( float scale );
	 void locateChildren( bool compact );
	 void moveChildren(void);
	 void connectChildren(void);
	 void makePixmaps( float scale );
	 void mousePressEvent( QMouseEvent *me );
	 void mouseReleaseEvent( QMouseEvent *me );
	 void mouseMoveEvent( QMouseEvent *me );
	 void resetChildPalette(void);
	 bool isInBox( QPoint posn, QPoint corner, QSize range );
	 //
	 QeLCDNumber *track, *time;
	 QeLabel *titlename, *trackname;
	 //
	 QeLabel *titleboxlabel, *timelabel, *stateicon, *modeicon;
	 QMeter *volm;
	 //
	 QSize df_s, track_s, time_s, title_s, trackname_s;
	 QSize bl_title_s, bl_time_s, stateicon_s, modeicon_s, vol_s;
	 QPoint df_p, track_p, time_p, title_p, trackname_p;
	 QPoint bl_title_p, bl_time_p, stateicon_p, modeicon_p, vol_p;
	 //
	 QColor childfg;
	 PixData pause_pd, play_pd, repeat_pd, rep1_pd;
	 QPixmap pause_pm, play_pm, blank_pm;
	 QPixmap rep_pm, rep1_pm;
	 int oldpmode;
	 // Only static const members can be initialized.  BUT - this
	 // member will be *identical* for all objects declared under
	 // this class because of the "static"!
	 static const int bsep=WIDGET_SEP;
};




//----------------------------------------------------------
//
// Inlined methods.
//
//----------------------------------------------------------




// Moves the entire display widget, including the children.
//
inline void pDisplay::moveAll( int x, int y )
{
  df_p.setX( x );
  df_p.setY( y );
  move( df_p + QPoint(bsep,bsep) );
}


// Overloaded version of the other function.
//
inline void pDisplay::moveAll( QPoint pos )
{
  df_p = pos;
  move( df_p + QPoint(bsep,bsep) );
}


// Returns a point specifying where to put the next widget
//
inline QPoint pDisplay::xEndPt()
{
  return( QPoint(df_s.width()+2*bsep,0) + df_p );
}


// Returns a point specifying where to put the next widget
//
inline QPoint pDisplay::yEndPt()
{
  return( QPoint(0,df_s.height()+2*bsep) + df_p );
}


// Changes the title box label to show the title string.
//
inline void pDisplay::useTitleLabel(void)
{
   titleboxlabel->setText( LABEL_TITLE );
}


// Changes the title box label to show the title string.
//
inline void pDisplay::useArtistLabel(void)
{
   titleboxlabel->setText( LABEL_ARTIST );
}






#endif // _pDisplay_h_
//||||||||||||||||||||||||||||||||
//
// End
// 
