/****************************************************************
**
** qCd cd-player
**
****************************************************************/
// Copyright (C) 1997 by John P. Weiss
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the Artistic License, included as the file
// "LICENSE" in the source code archive.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
//
// You should have received a copy of the file "LICENSE", containing
// the License John Weiss originally placed this program under.

#include <stdio.h>
#include <string.h>
#include <qapp.h>
#include <qpalette.h>
#include "qCd.h"


struct CmdlOptions
{
	 bool hasGeom;
	 bool hasFont;
	 bool hasBgc;
	 bool hasFgc;
//	 int n_qtx=7;
//	 char *qtx[7]={"-geometry","-fn","-font","-bg",
//				"-background","-fg","-foreground"};
	 bool squish;
	 bool shrink;
//	 int n_native=2;
//	 char *native[2]={"-compact","-small"};
};



// Global Color Palette function
//
inline
QPalette appDefaultColors( void )
{
  // This one alters the color of an object.
  // The parameters are, in order:  foreground, background, light,
  // dark, mid, text, base
  // Can probably set text=foreground and base=background
  // light, dark, and mid are used for shadowing effects.
  // mid = shadow on a button; light = hilight on a button.
  QColorGroup colgrp( white, black, lightGray, darkGray.dark(150), 
				  darkGray.dark(160), white, black );
  return QPalette( colgrp, colgrp, colgrp );
}




//||||||||||||||||||||||||||||||||||||||||||||||||
//
//  main
//
//||||||||||||||||||||||||||||||||||||||||||||||||

int main( int argc, char **argv )

{
  int i;
  bool scrunch,big;
  QPalette defpal;

  // The main application
  QApplication mainapp( argc, argv );

  scrunch = false;
  big = true;
  for(i=1; i<argc; i++)
	{
	if(strncmp(argv[i],"-s",2)==0)
	   big = false;
	else if(strncmp(argv[i],"-c",2)==0)
	   scrunch = true;
	}


  // Creates the AudioPanel widget, making it the main widget.
  defpal =  appDefaultColors();
  AudioPanel mainwin( NULL, "qCd", big, scrunch );
  mainwin.setPanelIcon( cd_icon_xpm );
  mainwin.changePalette( defpal );
  mainapp.connect( &mainwin, SIGNAL(quit()),
  			    SLOT(quit()) );
  mainapp.setMainWidget( &mainwin );


  // Creates the config object and loads the rc-file
  Config rc;


  // Inits the drive and plugs in some of the drive-related slots.
  cout << "Initializing CDROM device." << endl;
  AudioCD cd;
  cd.TrayControl( rc.useIoctl, !(rc.hasTray) );
  mainwin.pluginAudioControl(&cd);
  mainwin.pluginDisplayChange(&cd);
  cd.connect( &mainwin, SIGNAL(loadeject()), SLOT(LoadOrEject()) );
  cd.DeviceFirstInit( rc.devname );


  // Inits the program object.
  Program progpanel(&cd);

  // Inits the database management objects.
  cout << "Initializing playlist databases.  Please wait." << endl;
  MaintainDB *dbpanel;
  if(rc.useWMDB && rc.useTKDB)
	{
	dbpanel = new MaintainDB( &cd, rc.wmdbname, rc.tkdbname );
	dbpanel->readFrom = dbtBoth;
	}
  else if(rc.useWMDB)
	{
	dbpanel = new MaintainDB( &cd, rc.wmdbname );
	dbpanel->readFrom = dbtWM;
	}
  else if(rc.useTKDB)
	{
	dbpanel = new MaintainDB( &cd, NULL, rc.tkdbname );
	dbpanel->readFrom = 	dbtTK;
	}
  else
	{
	dbpanel = new MaintainDB( &cd );
	dbpanel->readFrom = dbtNone;
	}
  dbpanel->saveTo = dbpanel->readFrom;
  dbpanel->connect( &cd, SIGNAL(checkProgramCatalog()),
			    SLOT(checkCatalog()) );
  dbpanel->checkCatalog();
  mainwin.pluginPanels(&progpanel,&rc,dbpanel);


  // The exec() member function in Class QApplication is what passes
  // control to Qt.  We also wait to show the mainwin until we're
  // ready to enter the main control loop.
  mainwin.show();
  return mainapp.exec();
}






//||||||||||||||||||||||||||||||||
//
// End
//
